#include <SoftwareSerial.h>   //کتابخانه سریال نرم افزاری جهت ارتباط GPS و آردوینو
SoftwareSerial gps(4, 7);   //تعیین پایه های ارتباط سریال، توجه: ورودی اول پایه RX ماژول ESP و ورودی دوم پایه TX آن را تعیین می کند.
#include <TinyGPS++.h>   //کتابخانه GPS
TinyGPSPlus gps_d;  //ایجاد یک نمونه شی از کلاس کتابخانه GPS
TinyGPSCustom pdop(gps_d, "GPGSA", 15); //  تعیین تعدیل دقت موقعیت، فیلد ۱۵ از پکت GPGSA مربوط به دقت موقعیت است.
TinyGPSCustom hdop(gps_d, "GPGSA", 16); // تعیین تعدیل دقت افقی، فیلد ۱۶ از پکت GPGSA مربوط به این دقت است.
TinyGPSCustom vdop(gps_d, "GPGSA", 17); //تعیین تعدیل دقت عمودی، فیلد ۱۷ از پکت GPSA مربوط به این دقت است.
//برای اطلاعات بیشتر در زمینه تعدیل دقت،  لینک زیر را مطالعه کنید.
//https://www.agsgis.com/What-is-PDOP-And-Why-its-Obsolete_b_43.html
//توجه داشته باشید که ماژول های GPS داده های خود را تحت فرمت NMEA ارسال می کنند. این فرمت که استاندارد نام گذاری نیروی دریایی ایالات متحده است، داده های دریافتی از ماهواره ها را
//به شکل قابل فهم برای کاربر تبدیل می کنند. برای اطلاعات بیشتر در این زمینه، می توانید لینک زیر را مطالعه کنید.
//https://www.gpsworld.com/what-exactly-is-gps-nmea-data/
#define relay 10
void setup()
{
  //Init serial
  Serial.begin(9600);   //تعیین باودریت
  check_connect();  // اتصال به شبکه این تابع منتطر اتصال ماژول sim800l به شبکه مامده و پس از اتصال، برنامه ادامه می یابد.
  Serial.print("AT+CMGF=1\r\n");   //به کمک این دستور ارسال پیامک در حالت متن قرار می گیرد.
  delay(1000);
  Serial.print("AT+CMGD=1,4\r\n");    //به کمک این دستور حافظه پیامک ها را خالی می کنیم.
  delay(1000);
  Serial.print("AT+CSMP=17,167,0,0\r\n");   //به کمک این دستور تنظیمات ارسال پیامک را انجام می دهیم. این دستور برای سیم کارت ایرانسل الزامی است.
  delay(1000);
  gps.begin(9600);  //راه اندزای سریال نرم افزاری، توجه: باودریت ماژول GPS در حالت عادی ۹۶۰۰ است.
  delay(2500);
  pinMode(relay, OUTPUT);

}

void loop()
{
  send_data(); //تابع جهت تشخیص تماس تلفنی و ارسال داده ها
  smartDelay(100); //تابع جهت خواندن داده های GPS، در این تابع مدت زمان جهت انتظار برای خواندن داده های جدید را در ورودی به میلی ثانیه قرار می دهیم.
  relay_ctrl();
}
void check_connect() {
  char ccalr[] = {"+CCALR: 1"};
  Serial.println("WAITING TO CONNECT TO NETWORK");
  Serial.flush();
  while (true) {
    Serial.print("AT+CCALR?\r\n"); // ارسال دستور AT+CCALR به ماژول، در صورتیکه ماژول به شبکه متصل گردد ماژول عبارت CCALR: 1 را بر میگرداند
    if (Serial.find(ccalr) == true) // نتیجه تابع find در صورتی true می شود که عبارت مورد نظر یافت شده باشد، در غیر اینصورت درون حلقه تا زمانیکه نتیجه مورد نظر یافت نشود، باقی خواهد ماند
      break; // خروج از حلقه با یافتن پیام مورد نظر
  }
  Serial.println("CONNECTED TO NETWORK");
  Serial.flush();
}
void send_data() {
  char RING[] = {"RING"};  // زمانیکه که با شماره سیم کارت درون ماژول تماس گرفته شود، ماژول عبارت RING را بر می گرداند.
  String number = "";
  String data = "";
  if (Serial.find(RING) == true) {   //در صورتیکه تابع find عبارت RING را در بافر سریال بیابد، یعنی با ماژول تماس گرفته شده است.
    delay(500);
    Serial.print("ATH\r\n");//قطع تماس تلفنی
    delay(1000);
    send_sms("DATA:");   //تابع send_sms مخصوص ارسال پیام است. ورودی این تابع متن پیام را قبول کرده و سایر داده ها نظیر سرت و لینک موقعیت جغرافیایی را پیامک می کند. در این جا متن ابتدایی پیامک را DATA قرار می دهیم
  }
}
static void smartDelay(unsigned long ms)
{
  unsigned long start = millis();
  do
  {
    while (gps.available())   //تا زمانیکه در بافر داده وجود دارد
      gps_d.encode(gps.read());  //داده ها را بخوان
  } while (millis() - start < ms);   //مدت زمان جهت خواندن بافر ما در اینجا ۵۰۰ میلی ثانیه را تعیین کرده ایم.
}

void send_sms( String data) {
  //توجه: مطابق فرمت زیر، داده های در پیامک با ویرگول از هم جدا می شوند. به عنوان مثال داده ها به صورت زیر خواهند بود
  String number = "";
  unsigned long tm = 0;
  char cmgs[] = {"+CMGS:"}; // متغیر جهت کنترل ارسال پیام متنی
  if (Serial.available () > 0)
    Serial.readString();
  String cbc = ""; //متغیر جهت ذخیره شارژ باتری
  Serial.print("AT+CBC\r\n");   //دستور جهت دریافت میزان شارژ باتری
  delay(1500);
  cbc = Serial.readString();
  cbc = cbc.substring(cbc.indexOf(",") + 1);   //جداسازی داده ها جهت رسیدن به میزان شارژ باتری برای اطلاعات بیشتر، لینک زیر را مطالعه کنید.
  //https://digispark.ir/practical-at-commands-in-sim800l-gsm-module/
  cbc = cbc.substring(0, cbc.indexOf(","));
  data = data + "\nCURRENT SPEED:";
  data = data + String(gps_d.speed.kmph());  //دریافت سرعت
  //************************************************
  data = data + "\nBATTEY CHARGE:";
  data = data + cbc;
  //********************************************************
  data = data + "\n";  //یک خط پایین تر می بریم تا داده های GPS را به همراه لینک گوگل مپ قرار دهیم.
  //*************************************************
  data = data + "google.com/maps?q=";  //برای نمایش داده در گوگل مپ، می بایست پس از عبارت رو به رو، طول و عرض جغرافیایی را قرار داده و این دو مقدار را با یک ویرگول از هم جدا کنیم.
  data = data + String(gps_d.location.lat(), 6);  //دریافت عرض جغرافیایی
  data = data + ",";
  data = data + String(gps_d.location.lng(), 6);  //دریافت طول جغرافیایی با ۶ رقم دقت اعشار
  //**************************************************
  number = "+98*************";//در اینجا به جای ستاره ها شماره تماس خود را قرار دهید. به این شماره داده ها ارسال می گردد.
  Serial.print("AT+CMGS=\"" + number + "\"\r\n");
  Serial.flush();
  delay(1000);
  Serial.println(data);//ارسال متن پیام که اطلاعات بیمار به همراه موقعیت جغرافیایی است
  Serial.flush();
  delay(1000);
  Serial.write(0x1a);  //کاراکتر ctrl+z جهت ارسال پیام
  delay(1000);
  tm = millis();
  while (Serial.find(cmgs) == false && (millis() - tm) < 20000)   // دراین قسمت چنانچه پیامک ارسال شود عبارت CMGS+ از سمت ماژول دریافت می گردد. برای مدت ۲۰ ثانیه صبر می کنیم. چنانچه این عبارت دریافت نشد، از حلقه خارج می شویم.
    delay(50);
  Serial.print("AT+CSCLK=2");  //قرارگیری ماژول سیم کارت در حالت کم مصرف
  delay(2500);
}
void relay_ctrl() {
  String data="";
  char CMTI[] = {"+CMTI"};  //در صورت دریافت پیامک این عبارت توسط ماژول ارسال می شود.
  if (Serial.find(CMTI) == true) {  //در صورت دریافت پیامک
    if (Serial.available() > 0)  //پاک کردن بافر سریال از داده های پیشین
      Serial.readString();
    Serial.print("AT+CMGR=1\r\n");  //خواندن متن پیام از خانه شماره 1 حافظه سیم کارت همیشه پس از دریافت پیامک حافظه را پاک می کنیم تا پیام تنها در خانه شماره یک حافظه باشد.
    while (Serial.available() == 0) { ///انتظا ربرای دریافت پاسخ که متن پیامک در آن قرار دارد.
      delay(20);
    }
    data = Serial.readString();
    if (data.indexOf("ON10") >= 0)  // با عبارت ON10 رله فعال می شود.
      digitalWrite(relay, HIGH);
    else if (data.indexOf("OFF10") >= 0)   // با عبارت OFF10 رله خاموش می شود.
      digitalWrite(relay, LOW);
    Serial.print("AT+CMGD=1,4\r\n"); //پاک کردن پیام های حافظه سیم کارت
    delay(1000);
  }
}
